/**
 * Copyright 2017 ACCELECOM LTD.  All Rights Reserved.
 *  @file userdemo.cpp
 *  @company 南京艾科朗克信息科技有限公司
 *  @author shuaiw
 *  @modify changfengc
 *  @source encoding UTF-8
 *  @console encoding GB2312
 *  @Formatter GNU Built-in
 *  @brief 一个简单的例子，介绍CXeleTraderApi和CXeleTraderSpi接口的使用。
 *  本例将演示4个业务的基本流程,以及以下查询的结构体构建：
 *    1. 持仓查询(Position)
 *    2. 报单查询(Order)
 *    3. 成交单查询(Trade)
 *    4. 资金查询(Account)
 *   注意: 配置文件config_userdemo.ini最多可以配置两个交易用户(Trader 0和Trader 1)的信息
 *        默认操作只用Trader 0进行操作。
 */
#include <iostream>
#include <iomanip>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <assert.h>
#include <CXeleTraderApi.hpp>
#include <sched.h>
#include <unistd.h>

using namespace std;

#define MAX_ORDER_NUM 16

/* 求结构体成员字节大小 */
#define MEMB_SIZEOF(st_type, member) sizeof(((st_type*)0)->member)
/* 求结构体成员字节偏移 */
#define MEMB_CHARPTR(ptr, st_type, member) ((char*)ptr + offsetof(st_type, member))
/* 辅助宏: 打印marketData域字段 */
#define M_INPUT(ptr, CField, member, src) memcpy(MEMB_CHARPTR(ptr, CField, member), src, MEMB_SIZEOF(CField, member))
#define S_INPUT(ptr, CField, member, src) strncpy(MEMB_CHARPTR(ptr, CField, member), src, MEMB_SIZEOF(CField, member))
#define STR(x) #x

/* 响应信息 */
#define PRINT_RSP(RSPINFO)  fprintf(stdout, "\n{%s} Pass ErrorID:%d, ErrorMsg:%s nRequestID:%d Chain:%d\n\n", __FUNCTION__, \
		         RSPINFO->ErrorID, RSPINFO->ErrorMsg, nRequestID, bIsLast)

#define PRINT_RSP_ERR(RSPINFO)  fprintf(stdout, "\n{%s} Fail ErrorID:%d, ErrorMsg:%s nRequestID:%d Chain:%d\n\n", __FUNCTION__, \
		         RSPINFO->ErrorID, RSPINFO->ErrorMsg, nRequestID, bIsLast)

#define MAX_TRADERS 2


/*
 * 辅助函数: 读取配置文件*.ini
 */
static char g_Password_str[MAX_TRADERS][MEMB_SIZEOF(CXeleFtdcReqUserLoginField,Password)];
static char g_AccountID_str[MAX_TRADERS][MEMB_SIZEOF(CXeleFtdcQryClientAccountField, AccountID)];
static char g_FrontAddress_str[80];
static char g_LocalAddress_str[80];
static char g_QueryFrontAddress_str[80];
static char g_AuthInfo_str[MAX_TRADERS][MEMB_SIZEOF(CXeleFtdcAuthenticationInfoField,AuthCode)];
static char g_APPID_str[MAX_TRADERS][MEMB_SIZEOF(CXeleFtdcAuthenticationInfoField,AppID)];

static int iniLoad(char* buf, const char* item, const char* iniName)
{
    static const char sed_fmt[] = "sed 's/%s=\\(.*\\)/\\1/g;tx;d;:x' %s";
    char shell_cmd[160];
    FILE* ini;

    sprintf(shell_cmd, sed_fmt, item, iniName);
    ini = popen(shell_cmd, "r");
    fscanf(ini, "%s\n", buf);
    pclose(ini);
    return 0;
}

static void loadConfigFile(char* iniName)
{
    char buf[32];

    if (iniName == NULL || iniName[0] == 0)
    {
        return;
    }

    for (int i = 0; i < MAX_TRADERS; ++i)
    {
        sprintf(buf, "PASSWD%d", i);
        memset(g_Password_str[i], 0, sizeof(g_Password_str[i]));
        iniLoad(g_Password_str[i], buf, iniName);
        fprintf(stdout, "%s:%s\n", buf, g_Password_str[i]);

        sprintf(buf, "ACCOUNTID%d", i);
        memset(g_AccountID_str[i], 0, sizeof(g_AccountID_str[i]));
        iniLoad(g_AccountID_str[i], buf, iniName);
        fprintf(stdout, "%s:%s\n", buf, g_AccountID_str[i]);

        sprintf(buf, "AUTHCODE%d", i);
        memset(g_AuthInfo_str[i], 0, sizeof(g_AuthInfo_str[i]));
        iniLoad(g_AuthInfo_str[i], buf, iniName);
        fprintf(stdout, "%s:%s\n", buf, g_AuthInfo_str[i]);

        sprintf(buf, "APPID%d", i);
        memset(g_APPID_str[i], 0, sizeof(g_APPID_str[i]));
        iniLoad(g_APPID_str[i], buf, iniName);
        fprintf(stdout, "%s:%s\n", buf, g_APPID_str[i]);
    }

    memset(g_FrontAddress_str, 0, sizeof(g_FrontAddress_str));
    memset(g_LocalAddress_str, 0, sizeof(g_LocalAddress_str));

    iniLoad(g_FrontAddress_str, "FRONTADDRESS", iniName);
    iniLoad(g_QueryFrontAddress_str, "QUERYFRONTADDRESS", iniName);
    iniLoad(g_LocalAddress_str, "LOCALADDRESS", iniName);
    fprintf(stdout, "FRONTADDRESS:%s\n", g_FrontAddress_str);
    fprintf(stdout, "QUERYFRONTADDRESS:%s\n", g_QueryFrontAddress_str);
    fprintf(stdout, "LOCALADDRESS:%s\n", g_LocalAddress_str);
}

/*
 *
 * 辅助函数: 填写login域
 */
static void example_ReqUserLogin(CXeleFtdcReqUserLoginField *req, size_t trader = 0)
{
    memset(req, 0, sizeof(*req));

    snprintf(req->AccountID, sizeof(req->AccountID), "%s", g_AccountID_str[trader]);
    snprintf(req->Password, sizeof(req->Password), "%s", g_Password_str[trader]);
}

static void example_RegisterAuthentication(CXeleFtdcAuthenticationInfoField *authinfo, size_t trader = 0)
{
    memset(authinfo, 0, sizeof(*authinfo));

    snprintf(authinfo->AuthCode, sizeof(authinfo->AuthCode), "%s", g_AuthInfo_str[trader]);
    snprintf(authinfo->AppID, sizeof(authinfo->AppID), "%s", g_APPID_str[trader]);
}


/*
* 构造报单
*/
static void example_makeup_order(CXeleFairInputOrderField *order, size_t trader, uint8_t client_index, uint16_t token)
{
    order->OrderLocalNo = 1;
    order->LimitPrice   = 35960;
    order->ClientIndex  = client_index;
    order->ClientToken  = token;
    /* 合约代码 */
    strcpy(order->InstrumentID, "cu1705");
    order->VolumeTotalOriginal  = 5;
    order->InsertType   = 0x1;
    order->MinVolume    = 25;
    order->ExchangeFront= 0;
}



/*
* 构造撤单
*/
static void example_makeup_action(CXeleFairOrderActionField *action, size_t trader, uint8_t client_index,
                                  uint16_t token, int order_sysno)
{
    action->OrderSysNo   = order_sysno;
    action->ClientIndex  = client_index;
    action->ClientToken  = token;
    action->ActionFlag   = '0';
}

/*
 * 批量委托单，构造报单
 */
static void example_makeup_order_batch(CXeleFairInputOrderField *order,
                                       uint8_t client_index,
                                       uint16_t token,
                                       int request_id,
                                       int order_local_no)
{
    order->ProductReserve1 = 0x65;
    order->ProductReserve3 = request_id;
    order->OrderLocalNo = order_local_no;
    order->LimitPrice   = 99.4;
    order->ClientIndex  = client_index;
    order->ClientToken  = token;
    strcpy(order->InstrumentID, "au2104");
    order->VolumeTotalOriginal  = 10;
    order->InsertType   = 0x2;
    order->MinVolume    = 25;
    order->ExchangeFront= 0;
}

/*
 *批量委托单，构造撤单
 */
static void example_makeup_action_batch(CXeleFairOrderActionField *action,
                                        uint8_t client_index,
                                        uint16_t token,
                                        int order_sysno,
                                        int request_id)
{
    action->ProductReserve1 = 0x67;
    action->ClientIndex  = client_index;
    action->ClientToken  = token;
    action->ProductReserve3 = request_id;
    action->ActionFlag = '0';
    action->OrderSysNo   = order_sysno;
}

/*
 * @class CSimpleOrderManager
 * @brief 利用报单管理场景，演示本API接口调用。
 * @notes CXeleTraderSpi中OnXXX()形式的方法为回调函数，不能阻塞过长时间,
 *        否则，会影响接收其他来自交易柜台的消息。
 */
class CSimpleOrderManager: public CXeleTraderSpi
{
public:
    CXeleFtdcReqUserLoginField login_info;
    CXeleFtdcAuthenticationInfoField auth_info;
    /* 构造函数，需要一个有效的指向CXeleTraderApi实例的指针 */
    CSimpleOrderManager(CXeleTraderApi* pTraderApi) :m_pTraderApi(pTraderApi)
    {
    }

    ~CSimpleOrderManager()
    {
    }

    /* 当客户端程序与交易柜台建立起通信连接，客户端程序需要进行登录 */
    virtual void OnFrontConnected()
    {
        fprintf(stdout, "{%s}\n", __FUNCTION__);
        example_ReqUserLogin(&login_info);
        m_RspEvent = false;
        example_RegisterAuthentication(&auth_info);
        m_pTraderApi->RegisterAuthentication(&auth_info);
        m_pTraderApi->ReqUserLogin(&login_info, 0);
    }

    /* 当客户端程序与交易柜台通信连接断开时，该方法被调用 */
    virtual void OnFrontDisconnected(int nReason)
    {
        fprintf(stdout, "OnFrontDisconnected:%d\n", nReason);
    }

    /* 当客户端程序发出登录请求之后，该方法会被调用，通知客户端程序登录是否成功  */
    virtual void OnRspUserLogin(CXeleFtdcRspUserLoginField *pRspUserLogin, CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {

        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
            m_exchange_num    = pRspUserLogin->ExchangeNum;
            m_client_index[0] = pRspUserLogin->ClientIndex[0];
            m_token[0]        = pRspUserLogin->Token[0];
            m_client_index[1] = pRspUserLogin->ClientIndex[1];
            m_token[1]        = pRspUserLogin->Token[1];
            fprintf(stdout, "OnRspUserLogin:ExchangeNum=%d, ClientIndex0=%d, Token0=0x%x, ClientIndex1=%d, Token1=0x%x\n", \
      m_exchange_num, m_client_index[0], m_token[0], m_client_index[1], m_token[1]);
        }

        m_RspEvent = true;
    }

    virtual void OnRspOrderInsert(CXeleFtdcInputOrderField *pInputOrder, CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
            if(m_order_cnt < m_exchange_num) {
                m_ordersysno[m_order_cnt++] = pInputOrder->OrderSystemNo;
            }
        }

        m_RspEvent = true;
    }

    virtual void OnRspOrderAction(CXeleFtdcOrderActionField *pOrderAction, CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
        }

        m_RspEvent = true;
    }

    /* 报单回报 */
    virtual void OnRtnOrder(CXeleFtdcOrderField *pOrder)
    {
        printf("OnRtnOrder: OrderLocalNo=[%d]\n", pOrder->OrderLocalNo);
    }

    /* 针对用户请求的出错通知 */
    virtual void OnRspError(CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {
        PRINT_RSP(pRspInfo);
        /* 客户端程序需进行错误处理 */
        exit(-1);
    }

    /* 持仓查询请求 */
    void ReqQryClientPosition(size_t trader = 0)
    {
        CXeleFtdcQryClientPositionField qryPosition;
        memset(&qryPosition, 0, sizeof(CXeleFtdcQryClientPositionField));

        /* 客户代码 */
        snprintf(qryPosition.AccountID, sizeof(qryPosition.AccountID), "%s",g_AccountID_str[trader]);
        /* 合约代码 */
        // strcpy (qryPosition.InstrumentID, "cu1705");
        m_RspEvent = false;

        m_pTraderApi->ReqQryClientPosition(&qryPosition, 1);
    }

    /* 持仓查询应答 */
    virtual void OnRspQryClientPosition( CXeleFtdcRspClientPositionField *pRspClientPosition, CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            PRINT_RSP(pRspInfo);
        }

        m_RspEvent = true;
    }

    /* 资金查询请求 */
    void ReqQryClientAccount(size_t trader = 0)
    {
        CXeleFtdcQryClientAccountField qryAccount;
        memset(&qryAccount, 0, sizeof(CXeleFtdcQryClientAccountField));

        /* 资金帐号 */
        snprintf(qryAccount.AccountID, sizeof(qryAccount.AccountID), "%s", g_AccountID_str[trader]);
        m_RspEvent = false;

        m_pTraderApi->ReqQryClientAccount(&qryAccount, 1);
    }

    /* 资金查询应答 */
    virtual void OnRspQryClientAccount(CXeleFtdcRspClientAccountField *pClientAccount, CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
        }

        m_RspEvent = true;
    }

    /* 报单查询请求 */
    void ReqQryOrder(size_t trader = 0)
    {
        CXeleFtdcQryOrderField qryOrder;
        memset(&qryOrder, 0, sizeof(CXeleFtdcQryOrderField));
        /* 客户代码 */
        snprintf(qryOrder.AccountID, sizeof(qryOrder.AccountID), "%s", g_AccountID_str[trader]);
        m_RspEvent = false;

        m_pTraderApi->ReqQryOrder(&qryOrder, 1);
    }

    /* 报单查询应答 */
    virtual void OnRspQryOrder(CXeleFtdcOrderField* pOrderField, CXeleFtdcRspInfoField* pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
        }

        m_RspEvent = true;
    }

    /* 成交单查询请求 */
    void ReqQryTrade(size_t trader = 0)
    {
        CXeleFtdcQryTradeField qryTrade;
        memset(&qryTrade, 0, sizeof(CXeleFtdcQryTradeField));
        /* 客户代码 */
        snprintf(qryTrade.AccountID, sizeof(qryTrade.AccountID), "%s", g_AccountID_str[trader]);
        /* 合约代码 */
        strcpy(qryTrade.InstrumentID, "cu1705");
        /* 成交编号 */
        strcpy(qryTrade.TradeID, "233");
        m_RspEvent = false;

        m_pTraderApi->ReqQryTrade(&qryTrade, 1);
    }

    /* 成交单查询应答 */
    virtual void OnRspQryTrade(CXeleFtdcTradeField* pTradeField, CXeleFtdcRspInfoField* pRspInfo, int nRequestID, bool bIsLast)
    {

        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
        }

        m_RspEvent = true;
    }

    /* 交易所前置查询请求 */
    void ReqQryExchangeFront()
    {
        m_RspEvent = false;
        m_pTraderApi->ReqQryExchangeFront(1);
    }

    /* 交易所前置查询应答 */
    virtual void OnRspQryExchangeFront(CXeleFtdcRspExchangeFrontField *pRspExchangeFront, CXeleFtdcRspInfoField* pRspInfo, int nRequestID, bool bIsLast)
    {
        if (pRspInfo->ErrorID != 0)
        {
            /* 失败， 客户端程序需进行错误处理  */
            PRINT_RSP_ERR(pRspInfo);
            exit(pRspInfo->ErrorID);
        }
        else
        {
            /* 成功，处理数据...  */
            PRINT_RSP(pRspInfo);
        }
        m_RspEvent = true;
    }

    /* 合约交易状态通知回报 */
    virtual void OnRtnInstrumentStatus(CXeleFtdcInstrumentStatusField* pInstrumentStatus)
    {
        fprintf(stdout, "\n{%s} InstrumentID=[%s],InstrumentStatus=[%c]\n", __FUNCTION__,
                pInstrumentStatus->InstrumentID, pInstrumentStatus->InstrumentStatus);
    }

    /* 等待响应事件 */
    void WaitRspEvent()
    {
        do
        {
            sched_yield();
        } while (true != m_RspEvent);
    }

    /* 测试流程：风控管理 */
    bool SimpleQueryManagement()
    {
        sleep(2);
        /* 交易所前置查询 */
        ReqQryExchangeFront();
        WaitRspEvent();

        sleep(2);
        /* 资金查询应答 */
        ReqQryClientAccount();
        WaitRspEvent();
        /*
        * 配置两个交易用户 trader=1)
        * ReqQryClientAccount (1);
        * WaitRspEvent ();
        */

        sleep(2);
        /* 持仓查询应答 */
        ReqQryClientPosition();
        WaitRspEvent();
        /*
        * 配置两个交易用户 trader=1)
        * ReqQryClientPosition (1);
        * WaitRspEvent ();
        */

        sleep(2);
        /* 成交单查询请求 */
        ReqQryTrade();
        WaitRspEvent();
        /*
        * 配置两个交易用户 trader=1)
        * ReqQryTrade (1);
        * WaitRspEvent ();
        */

        sleep(2);
        /* 报单查询请求 */
        ReqQryOrder();
        WaitRspEvent();
        /*
        * 配置两个交易用户 trader=1)
        * ReqQryOrder (1);
        * WaitRspEvent ();
        */

        return true;
    }

    /* 测试流程：报单管理 */
    void SimpleOrderManagement()
    {
        int i = 0;
        CXeleFairInputOrderField order;
        CXeleFairOrderActionField action;
        CXeleFairOrderFieldV order_action[MAX_ORDER_NUM];

        /* 等待登入成功 */
        WaitRspEvent();

        /* 查询接口 */
        (void)SimpleQueryManagement();

        /* 报单接口 */
        m_order_cnt = 0;
        for(i = 0; i < m_exchange_num; i++)
        {
            memset(&order, 0, sizeof(CXeleFairInputOrderField));
            example_makeup_order(&order, 0, m_client_index[i], m_token[i]);
            sleep(2);
            m_pTraderApi->ReqOrderInsert(&order, 1);
            WaitRspEvent();
        }

        /* 撤单接口 */
        for(i = 0; i < m_exchange_num; i++)
        {
            memset(&action, 0, sizeof(CXeleFairOrderActionField));
            example_makeup_action(&action, 0, m_client_index[i], m_token[i], m_ordersysno[i]);
            sleep(2);
            m_pTraderApi->ReqOrderAction(&action, 1);
            WaitRspEvent();
        }

        /* 批量委托单接口*/
        for(i = 0; i < m_exchange_num; i++) {
            int request_id = 100;
            int local_id = 1;
            int j = 0;

            memset(order_action, 0, sizeof(order_action));

            order_action[j].order.input.ProductReserve1 = 0x65;
            order_action[j].order.input.ProductReserve3 = request_id++;
            order_action[j].order.input.OrderLocalNo = local_id++;
            order_action[j].order.input.LimitPrice = 99.4;
            order_action[j].order.input.ClientIndex = m_client_index[i];
            order_action[j].order.input.ClientToken = m_token[i];
            strcpy(order_action[j].order.input.InstrumentID, "au2104");
            order_action[j].order.input.VolumeTotalOriginal = 10;
            order_action[j].order.input.InsertType = 0x2;
            order_action[j].order.input.MinVolume = 25;
            order_action[j].order.input.ExchangeFront = 0;
            j++;

            order_action[j].order.action.ProductReserve1 = 0x67;
            order_action[j].order.action.ClientIndex = m_client_index[i];
            order_action[j].order.action.ClientToken = m_token[i];
            order_action[j].order.action.ProductReserve3 = request_id++;
            order_action[j].order.action.ActionFlag = '0';
            //ordersysno
            order_action[j].order.action.OrderSysNo = 1;
            j++;

            sleep(2);
            m_pTraderApi->ReqOrderInsertActionV(order_action, j);
            WaitRspEvent();
        }
    }

private:
    /* 指向CXeleTraderApi实例的指针 */
    CXeleTraderApi* m_pTraderApi;

    /* 响应事件标志 */
    bool m_RspEvent;
    uint32_t m_order_cnt;
    int m_ordersysno[XELE_EXCHANGE_LOGIN_NUM];
    uint8_t m_exchange_num;
    uint8_t m_client_index[XELE_EXCHANGE_LOGIN_NUM];
    uint16_t m_token[XELE_EXCHANGE_LOGIN_NUM];
};

/*
 * 主函数:
 */
int main(int argc, char* argv[])
{
    /*
     * 读取config_userdemo.ini
     */
    loadConfigFile((char*) "config_userdemo.ini");

    std::string msg;
    CXeleTraderApi* pTraderApi = CXeleTraderApi::CreateTraderApi(0);
    assert(pTraderApi);
    CSimpleOrderManager som(pTraderApi);
    pTraderApi->RegisterSpi(&som);
    pTraderApi->RegisterFront(g_FrontAddress_str, g_QueryFrontAddress_str, g_LocalAddress_str);
    /* 默认报单通讯是非阻塞模式，若设置阻塞模式请在调用Init函数之前设置 */
    /* 设置为阻塞模式 */
    pTraderApi->RegisterChannelBlock(1);

    /*
     * 订阅相应的流
     */
    pTraderApi->SubscribePrivateTopic(XELE_TERT_RESTART);
    pTraderApi->SubscribePublicTopic(XELE_TERT_RESTART);

    /*
     * 开始登录, 使客户端程序开始与交易柜台建立连接
     */
    pTraderApi->Init();

    /*
     * 打印API信息
     */
    fprintf(stdout, "%s\n", pTraderApi->GetVersion());

    /*
     * 实例流程：查询管理
     */
    som.SimpleOrderManagement();

    /*
     * 主线程打印, 等待退出
     */
    do
    {
        cout << "Input 'q' to disconnect API:";
        getline(cin, msg);
    } while (msg != "q");

    /*
     * 释放API实例
     */
    pTraderApi->Release();
    fprintf(stdout, "API release done. Exit Demo.\n");

    return 0;
}
